<!doctype html>
<html>
<head>
    <title>Golem keywords</title>
</head>
<body>
<h1>Keywords for entity {{ entity }}</h1>
<table>
    <tr>
        <td>Value</td>
        <td>Foo</td>
    </tr>
</table>
</body>
</html>