# Generated by Django 2.0.2 on 2018-06-05 10:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('golem', '0003_auto_20180207_2331'),
    ]

    operations = [
        migrations.CreateModel(
            name='WebMessageData',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uid', models.TextField()),
                ('is_response', models.BooleanField()),
                ('timestamp', models.DateTimeField(auto_now=True)),
                ('data', models.TextField()),
            ],
        ),
    ]
