# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-09 13:24
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('golem', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Chat',
            fields=[
                ('chat_id', models.CharField(max_length=255, primary_key=True, serialize=False)),
                ('user_uid', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='golem.User')),
            ],
        ),
    ]
