# Generated by Django 2.0.2 on 2018-02-07 22:31

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('golem', '0002_chat'),
    ]

    operations = [
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('text', models.TextField(blank=True, null=True)),
                ('is_from_user', models.BooleanField()),
                ('time', models.DateTimeField(auto_now=True)),
                ('intent', models.TextField(blank=True, db_index=True, max_length=255, null=True)),
                ('state', models.TextField(blank=True, db_index=True, max_length=255, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='QuickReply',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField(max_length=255)),
                ('message', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='golem.Message')),
            ],
        ),
        migrations.RemoveField(
            model_name='user',
            name='last_active',
        ),
        migrations.AddField(
            model_name='message',
            name='chat',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='golem.Chat'),
        ),
    ]
